@@ -1,6 +1,6 @@
 /*
  *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  Copyright (C) 1997-2005 Kay Roemer & Arno Puder
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -106,13 +106,13 @@ mkdep (const string &file, const svec &incdirs)
         }
         deps.insert (f);
 
-        while (!inp.getline (rawline, sizeof(rawline)).eof()) {
+        while (inp.getline (rawline, sizeof(rawline)).good()) {
             string line = rawline;
 	    bool real_path = false;
             if (line.length() == 0 || line[0] != '#')
                 continue;
             if ((long)line.find ("include") >= 0) {
-                long pos1, pos2;
+                long pos1, pos2 = 0;
                 pos1 = line.find ('<');
                 if (pos1 >= 0)
                     pos2 = line.find ('>', ++pos1);
