@@ -24,6 +24,9 @@
 #include <mico/impl.h>
 #include "NamingClient.h"
 
+
+using namespace std;
+
 #if 0
 #define TRY try {
 #define CATCH(__name) \
@@ -337,12 +340,12 @@ NamingClient::show_ior (const char* s)
 
 void printname(ostream & str,const CosNaming::Name& name)
 {
-  int i;
+  unsigned int i;
   for(i=0;i<name.length();i++)
   {
       if(i<0) str << '/';
-      str << name[i].id;
-      if(strlen(name[i].kind)>0) str << '.' << name[i].kind;
+      str << name[i].id.in();
+      if(strlen(name[i].kind.in())>0) str << '.' << name[i].kind.in();
   }
 }
 
