@@ -1,6 +1,6 @@
 /*
  *  Object Property Service for MICO
- *  Copyright (C) 1998 Leif Jakobsmeier
+ *  Copyright (C) 1998, 1999, 2000 Leif Jakobsmeier
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -20,8 +20,11 @@
  *                 mico@informatik.uni-frankfurt.de
  */
 
+#define MICO_CONF_POA
+#define MICO_CONF_INTERCEPT
+#define MICO_CONF_IR
 #include <CORBA-SMALL.h>
-#include <mico/PropertyService_impl.h>
+#include <coss/PropertyService_impl.h>
 
 #ifdef HAVE_EXCEPTIONS
 #define MULTIPLE_EXCEPTIONS_CATCH(__mul_exc, __fail_prop_name)                \
@@ -29,49 +32,57 @@
     CORBA::ULong len = __mul_exc.exceptions.length ();                        \
     __mul_exc.exceptions.length (len + 1);                                    \
     __mul_exc.exceptions[len].reason = PropertyService::invalid_property_name;\
-    __mul_exc.exceptions[len].failing_property_name = __fail_prop_name;       \
+    __mul_exc.exceptions[len].failing_property_name =                         \
+      CORBA::string_dup (__fail_prop_name);                                   \
   }                                                                           \
   catch (PropertyService::ConflictingProperty_catch &exp) {                   \
     CORBA::ULong len = __mul_exc.exceptions.length ();                        \
     __mul_exc.exceptions.length (len + 1);                                    \
     __mul_exc.exceptions[len].reason = PropertyService::conflicting_property; \
-    __mul_exc.exceptions[len].failing_property_name = __fail_prop_name;       \
+    __mul_exc.exceptions[len].failing_property_name =                         \
+      CORBA::string_dup (__fail_prop_name);                                   \
   }                                                                           \
   catch (PropertyService::PropertyNotFound_catch &exp) {                      \
     CORBA::ULong len = __mul_exc.exceptions.length ();                        \
     __mul_exc.exceptions.length (len + 1);                                    \
     __mul_exc.exceptions[len].reason = PropertyService::property_not_found;   \
-    __mul_exc.exceptions[len].failing_property_name = __fail_prop_name;       \
+    __mul_exc.exceptions[len].failing_property_name =                         \
+      CORBA::string_dup (__fail_prop_name);                                   \
   }                                                                           \
   catch (PropertyService::UnsupportedTypeCode_catch &exp) {                   \
     CORBA::ULong len = __mul_exc.exceptions.length ();                        \
     __mul_exc.exceptions.length (len + 1);                                    \
     __mul_exc.exceptions[len].reason = PropertyService::unsupported_type_code;\
-    __mul_exc.exceptions[len].failing_property_name = __fail_prop_name;       \
+    __mul_exc.exceptions[len].failing_property_name =                         \
+      CORBA::string_dup (__fail_prop_name);                                   \
   }                                                                           \
   catch (PropertyService::UnsupportedProperty_catch &exp) {                   \
     CORBA::ULong len = __mul_exc.exceptions.length ();                        \
     __mul_exc.exceptions.length (len + 1);                                    \
     __mul_exc.exceptions[len].reason = PropertyService::unsupported_property; \
-    __mul_exc.exceptions[len].failing_property_name = __fail_prop_name;       \
+    __mul_exc.exceptions[len].failing_property_name =                         \
+      CORBA::string_dup (__fail_prop_name);                                   \
   }                                                                           \
   catch (PropertyService::UnsupportedMode_catch &exp) {                       \
     CORBA::ULong len = __mul_exc.exceptions.length ();                        \
     __mul_exc.exceptions.length (len + 1);                                    \
     __mul_exc.exceptions[len].reason = PropertyService::unsupported_mode;     \
-    __mul_exc.exceptions[len].failing_property_name = __fail_prop_name;       \
+    __mul_exc.exceptions[len].failing_property_name =                         \
+      CORBA::string_dup (__fail_prop_name);                                   \
   }                                                                           \
   catch (PropertyService::FixedProperty_catch &exp) {                         \
     CORBA::ULong len = __mul_exc.exceptions.length ();                        \
     __mul_exc.exceptions.length (len + 1);                                    \
     __mul_exc.exceptions[len].reason = PropertyService::fixed_property;       \
-    __mul_exc.exceptions[len].failing_property_name = __fail_prop_name;       \
+    __mul_exc.exceptions[len].failing_property_name =                         \
+      CORBA::string_dup (__fail_prop_name);                                   \
   }                                                                           \
   catch (PropertyService::ReadOnlyProperty_catch &exp) {                      \
     CORBA::ULong len = __mul_exc.exceptions.length ();                        \
     __mul_exc.exceptions.length (len + 1);                                    \
     __mul_exc.exceptions[len].reason = PropertyService::read_only_property;   \
-    __mul_exc.exceptions[len].failing_property_name = __fail_prop_name;       \
+    __mul_exc.exceptions[len].failing_property_name =                         \
+      CORBA::string_dup (__fail_prop_name);                                   \
   }
 #else
 #define MULTIPLE_EXCEPTIONS_CATCH(__mul_exc, __fail_prop_name)
@@ -95,8 +106,9 @@ PropertySetFactory_impl::~PropertySetFactory_impl ()
 PropertyService::PropertySet_ptr
 PropertySetFactory_impl::create_propertyset ()
 {
-  return PropertyService::PropertySet::_duplicate 
-    (new PropertySet_impl ());
+  PropertySet_impl* pset = new PropertySet_impl ();
+  PropertyService::PropertySet_ptr pset_ref = pset->_this();
+  return pset_ref;
 }
 
 PropertyService::PropertySet_ptr
@@ -104,7 +116,7 @@ PropertySetFactory_impl::create_constrained_propertyset
                   (const PropertyService::PropertyTypes &allowed_property_types,
                    const PropertyService::Properties &allowed_properties)
 {
-  PropertyService::PropertySet_ptr new_ps;
+  PropertySet_impl* new_ps;
   #ifdef HAVE_EXCEPTIONS
   try {
   #endif
@@ -118,18 +130,17 @@ PropertySetFactory_impl::create_constrained_propertyset
     mico_throw (PropertyService::ConstraintNotSupported ());
   }
   #endif
-
-  return PropertyService::PropertySet::_duplicate (new_ps);
+  PropertyService::PropertySet_ptr ref = new_ps->_this();
+  return ref;
 }
 
 PropertyService::PropertySet_ptr
 PropertySetFactory_impl::create_initial_propertyset 
                    (const PropertyService::Properties &initial_properties)
 {
-  PropertyService::PropertySet_ptr new_ps;
-  new_ps = new PropertySet_impl (initial_properties);
-
-  return PropertyService::PropertySet::_duplicate (new_ps);
+  PropertySet_impl* new_ps = new PropertySet_impl (initial_properties);
+  PropertyService::PropertySet_ptr ref = new_ps->_this();
+  return ref;
 }
 
 /*****************************************************************************/
@@ -149,8 +160,9 @@ PropertySetDefFactory_impl::~PropertySetDefFactory_impl ()
 PropertyService::PropertySetDef_ptr
 PropertySetDefFactory_impl::create_propertysetdef ()
 {
-  return PropertyService::PropertySetDef::_duplicate 
-    (new PropertySetDef_impl ());
+  PropertySetDef_impl* psdef = new PropertySetDef_impl ();
+  PropertyService::PropertySetDef_ptr ref = psdef->_this();
+  return ref;
 }
 
 PropertyService::PropertySetDef_ptr
@@ -158,7 +170,7 @@ PropertySetDefFactory_impl::create_constrained_propertysetdef
                   (const PropertyService::PropertyTypes &allowed_property_types,
                    const PropertyService::PropertyDefs &allowed_property_defs)
 {
-  PropertyService::PropertySetDef_ptr new_psd;
+  PropertySetDef_impl* new_psd = NULL;
   #ifdef HAVE_EXCEPTIONS
   try {
   #endif
@@ -173,18 +185,17 @@ PropertySetDefFactory_impl::create_constrained_propertysetdef
     mico_throw (PropertyService::ConstraintNotSupported ());
   }
   #endif
-
-  return PropertyService::PropertySetDef::_duplicate (new_psd);
+  PropertyService::PropertySetDef_ptr ref = new_psd->_this();
+  return ref;
 }
 
 PropertyService::PropertySetDef_ptr
 PropertySetDefFactory_impl::create_initial_propertysetdef 
                    (const PropertyService::PropertyDefs &initial_property_defs)
 {
-  PropertyService::PropertySetDef_ptr new_psd;
-  new_psd = new PropertySetDef_impl (initial_property_defs);
-
-  return PropertyService::PropertySetDef::_duplicate (new_psd);
+  PropertySetDef_impl* new_psd = new PropertySetDef_impl (initial_property_defs);
+  PropertyService::PropertySetDef_ptr ref = new_psd->_this();
+  return ref;
 }
 
 /*****************************************************************************/
@@ -194,6 +205,7 @@ PropertySetDefFactory_impl::create_initial_propertysetdef
 /*****************************************************************************/
 
 PropertySet_impl::PropertySet_impl ()
+    : ps_lock_(FALSE, MICOMT::Mutex::Recursive)
 {
 }
 
@@ -202,6 +214,7 @@ PropertySet_impl::PropertySet_impl (const PropertyService::PropertyTypes
                                       &allowed_property_types,
                                     const PropertyService::Properties
                                       &allowed_properties)
+    : ps_lock_(FALSE, MICOMT::Mutex::Recursive)
 {
   CORBA::ULong len = allowed_property_types.length ();
 
@@ -241,6 +254,7 @@ PropertySet_impl::PropertySet_impl (const PropertyService::PropertyTypes
 /* Used by PropertySetFactory::create_initial_propertyset (...) */
 PropertySet_impl::PropertySet_impl (const PropertyService::Properties
                                       &initial_properties)
+    : ps_lock_(FALSE, MICOMT::Mutex::Recursive)
 {
   def_props (initial_properties, PropertyService::fixed_normal);
 }
@@ -258,6 +272,7 @@ CORBA::Boolean
 PropertySet_impl::get_index (const char *property_name,
                              CORBA::ULong *idx)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   CORBA::UShort max = get_number_of_properties ();
   for(CORBA::UShort i = 0; i < max; i++)
   {
@@ -274,6 +289,7 @@ PropertySet_impl::get_index (const char *property_name,
 PropertyService::PropertyName
 PropertySet_impl::get_name (CORBA::ULong idx)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   assert (idx < get_number_of_properties ());
   return mv_properties[idx]->property_name;
 }
@@ -281,6 +297,7 @@ PropertySet_impl::get_name (CORBA::ULong idx)
 CORBA::Any *
 PropertySet_impl::get_value (CORBA::ULong idx)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   assert (idx < get_number_of_properties ());
   return &mv_properties[idx]->property_value;
 }
@@ -297,6 +314,7 @@ CORBA::Boolean
 PropertySet_impl::is_property_type_allowed (CORBA::TypeCode_ptr 
                                             property_type)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   CORBA::ULong len = mv_allowed_property_types.size (); 
   
   if (len == 0)
@@ -323,6 +341,7 @@ PropertySet_impl::is_property_allowed (const char *name,
                                        const PropertyService::PropertyModeType 
                                          &mode_type)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   CORBA::ULong len = mv_allowed_properties.size ();
 
   if (len == 0)
@@ -351,6 +370,7 @@ PropertySet_impl::is_property_allowed (const char *name,
 CORBA::Boolean
 PropertySet_impl::is_property_readonly (CORBA::ULong idx)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   if (mv_properties[idx]->property_mode == PropertyService::read_only ||
       mv_properties[idx]->property_mode == PropertyService::fixed_readonly)
     return TRUE;
@@ -360,6 +380,7 @@ PropertySet_impl::is_property_readonly (CORBA::ULong idx)
 CORBA::Boolean
 PropertySet_impl::is_property_fixed (CORBA::ULong idx)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   if (mv_properties[idx]->property_mode == PropertyService::fixed_normal ||
       mv_properties[idx]->property_mode == PropertyService::fixed_readonly)
     return TRUE;
@@ -371,6 +392,7 @@ PropertySet_impl::def_prop (const char *property_name,
                             const CORBA::Any &property_value,
                             const PropertyService::PropertyModeType &mode_type)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   /* Check if PropertyName is valid */
   if (! is_property_name_valid (property_name)) {
     mico_throw (PropertyService::InvalidPropertyName ());
@@ -425,6 +447,7 @@ void
 PropertySet_impl::def_props (const PropertyService::Properties &nproperties,
                              const PropertyService::PropertyModeType &mode_type)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   CORBA::ULong len = nproperties.length();
   assert (len > 0); 
 
@@ -445,7 +468,7 @@ PropertySet_impl::def_props (const PropertyService::Properties &nproperties,
   }
 
   if (mul_exc.exceptions.length () != 0) {
-    mico_throw (PropertyService::MultipleExceptions (mul_exc));
+    mico_throw (mul_exc);
   }
 }
 
@@ -462,6 +485,7 @@ void
 PropertySet_impl::define_property (const char *property_name,
                                    const CORBA::Any &property_value)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   def_prop (property_name, property_value, PropertyService::normal);
 }
 
@@ -469,6 +493,7 @@ void
 PropertySet_impl::define_properties (const PropertyService::Properties 
                                      &nproperties)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   def_props (nproperties, PropertyService::normal);
 }
 
@@ -479,14 +504,16 @@ PropertySet_impl::define_properties (const PropertyService::Properties
 CORBA::ULong
 PropertySet_impl::get_number_of_properties ()
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   return mv_properties.size ();
 } 
 
 void
 PropertySet_impl::get_all_property_names (CORBA::ULong how_many, 
-                                PropertyService::PropertyNames *&property_names,
-                                PropertyService::PropertyNamesIterator *&rest)
+                                PropertyService::PropertyNames_out property_names,
+                                PropertyService::PropertyNamesIterator_out rest)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   CORBA::ULong len = get_number_of_properties ();
 
   property_names = new PropertyService::PropertyNames ();
@@ -503,19 +530,21 @@ PropertySet_impl::get_all_property_names (CORBA::ULong how_many,
   
   if (how_many >= len) {
     /* All Properties have been copyed => return an empty PropertyNamesIter. */
-    rest = PropertyService::PropertyNamesIterator::_duplicate 
-             (new PropertyNamesIterator_impl ());
+    PropertyNamesIterator_impl* iter = new PropertyNamesIterator_impl;
+    rest = iter->_this();
     return;
   }
   /* Initiate PropertyNamesIterator */
-  rest = PropertyService::PropertyNamesIterator::_duplicate 
-           (new PropertyNamesIterator_impl (this, how_many));
+  PropertyNamesIterator_impl* iter
+    = new PropertyNamesIterator_impl (this, how_many);
+  rest = iter->_this();
   return;
 }
 
 CORBA::Any *
 PropertySet_impl::get_property_value (const char *property_name) 
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   /* Check if PropertyName is valid */
   if (! is_property_name_valid (property_name)) {
     mico_throw (PropertyService::InvalidPropertyName ());
@@ -541,8 +570,9 @@ PropertySet_impl::get_property_value (const char *property_name)
 CORBA::Boolean
 PropertySet_impl::get_properties (const PropertyService::PropertyNames 
                                     &property_names,
-                                  PropertyService::Properties *&nproperties) 
+                                  PropertyService::Properties_out nproperties) 
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   CORBA::ULong len = property_names.length ();
   assert (len > 0);
   
@@ -578,15 +608,17 @@ PropertySet_impl::get_properties (const PropertyService::PropertyNames
 
     /* Copy Property_Value */ 
     (*nproperties)[idx].property_value = *property_value;
+    delete property_value;
   }
   return return_stat;
 }
 
 void
 PropertySet_impl::get_all_properties (CORBA::ULong how_many, 
-                                     PropertyService::Properties *&nproperties,
-                                     PropertyService::PropertiesIterator *&rest)
+                                     PropertyService::Properties_out nproperties,
+                                     PropertyService::PropertiesIterator_out rest)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   CORBA::ULong len = get_number_of_properties();
 
   /* Create new Properties Sequence */
@@ -607,13 +639,13 @@ PropertySet_impl::get_all_properties (CORBA::ULong how_many,
 
   if (how_many >= len) {
     /* All Properties have been copyed => return an empty PropertiesIterator */
-    rest = PropertyService::PropertiesIterator::_duplicate 
-             (new PropertiesIterator_impl ());
+    PropertiesIterator_impl* iter = new PropertiesIterator_impl ();
+    rest = iter->_this();
     return;
   }
   /* Initiate PropertiesIterator */
-  rest = PropertyService::PropertiesIterator::_duplicate 
-           (new PropertiesIterator_impl (this, how_many));
+  PropertiesIterator_impl* iter = new PropertiesIterator_impl (this, how_many);
+  rest = iter->_this();
   return;
 }
 
@@ -624,6 +656,7 @@ PropertySet_impl::get_all_properties (CORBA::ULong how_many,
 void
 PropertySet_impl::delete_property (const char *property_name)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   /* Check if PropertyName is valid */
   if (! is_property_name_valid (property_name)) {
     mico_throw (PropertyService::InvalidPropertyName ());
@@ -652,6 +685,7 @@ void
 PropertySet_impl::delete_properties (const PropertyService::PropertyNames 
                                       &property_names)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   CORBA::ULong len = property_names.length();
   assert (len > 0);   
 
@@ -670,32 +704,30 @@ PropertySet_impl::delete_properties (const PropertyService::PropertyNames
   }
  
   if (mul_exc.exceptions.length () != 0) {
-    mico_throw (PropertyService::MultipleExceptions (mul_exc));
+    mico_throw (mul_exc);
   }
 }
 
 CORBA::Boolean 
 PropertySet_impl::delete_all_properties()
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   CORBA::ULong len = get_number_of_properties();
 
-  CORBA::ULong pos = 0;
-  for (CORBA::ULong idx = 0; idx < len; idx++)
+  CORBA::Boolean success = TRUE;
+  for (CORBA::Long idx = len - 1; idx >= 0; idx--)
   {
     /* Check if Property is fixed */
-    if (! is_property_fixed (pos)) {
+    if (! is_property_fixed (idx)) {
       /* Delete Property */
-      mv_properties.erase (mv_properties.begin() + pos);
+      mv_properties.erase (mv_properties.begin() + idx);
     } else {
       /* Skip Property */
-      pos++;
+      success = FALSE;
     }
   }
  
-  if (pos == 0)
-    return TRUE;
-  else
-    return FALSE;
+  return success;
 }
 
   /************************************************/
@@ -705,6 +737,7 @@ PropertySet_impl::delete_all_properties()
 CORBA::Boolean 
 PropertySet_impl::is_property_defined (const char *property_name)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   /* Check if PropertyName is valid */
   if (! is_property_name_valid (property_name)) {
     mico_throw (PropertyService::InvalidPropertyName ());
@@ -786,9 +819,10 @@ PropertySetDef_impl::~PropertySetDef_impl ()
   /****************************************************/
 
 void
-PropertySetDef_impl::get_allowed_property_types (PropertyService::PropertyTypes
-                                                   *&property_types)
+PropertySetDef_impl::get_allowed_property_types (PropertyService::PropertyTypes_out
+                                                   property_types)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   CORBA::ULong len = mv_allowed_property_types.size ();
 
   property_types = new PropertyService::PropertyTypes;
@@ -799,9 +833,10 @@ PropertySetDef_impl::get_allowed_property_types (PropertyService::PropertyTypes
 }
 
 void
-PropertySetDef_impl::get_allowed_properties (PropertyService::PropertyDefs
-                                               *&property_defs)
+PropertySetDef_impl::get_allowed_properties (PropertyService::PropertyDefs_out
+                                               property_defs)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   CORBA::ULong len = mv_allowed_properties.size ();
 
   property_defs  = new PropertyService::PropertyDefs;
@@ -829,6 +864,7 @@ PropertySetDef_impl::define_property_with_mode
                         const CORBA::Any &property_value,
                         PropertyService::PropertyModeType property_mode)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   /* Check PropertyModeType */
   if (property_mode == PropertyService::undefined)
   {
@@ -841,6 +877,7 @@ void
 PropertySetDef_impl::define_properties_with_modes 
                        (const PropertyService::PropertyDefs &property_defs)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   CORBA::ULong len = property_defs.length();
   assert (len > 0);
 
@@ -861,7 +898,7 @@ PropertySetDef_impl::define_properties_with_modes
   }
 
   if (mul_exc.exceptions.length () != 0) {
-    mico_throw (PropertyService::MultipleExceptions (mul_exc));
+    mico_throw (mul_exc);
   }
 }
 
@@ -872,6 +909,7 @@ PropertySetDef_impl::define_properties_with_modes
 PropertyService::PropertyModeType
 PropertySetDef_impl::get_property_mode (const char *property_name)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   /* Check if PropertyName is valid */
   if (! is_property_name_valid (property_name)) {
     mico_throw (PropertyService::InvalidPropertyName ());
@@ -890,9 +928,10 @@ PropertySetDef_impl::get_property_mode (const char *property_name)
 CORBA::Boolean
 PropertySetDef_impl::get_property_modes (const PropertyService::PropertyNames
                                            &property_names,
-                                         PropertyService::PropertyModes 
-                                           *&property_modes)
+                                         PropertyService::PropertyModes_out 
+                                           property_modes)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   CORBA::ULong len = property_names.length ();
   assert (len > 0);
   
@@ -937,6 +976,7 @@ PropertySetDef_impl::set_property_mode (const char *property_name,
                                         PropertyService::PropertyModeType
                                           property_mode)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   /* Check if PropertyName is valid */
   if (! is_property_name_valid (property_name)) {
     mico_throw (PropertyService::InvalidPropertyName ());
@@ -969,6 +1009,7 @@ void
 PropertySetDef_impl::set_property_modes (const PropertyService::PropertyModes
                                            &property_modes)
 {
+  MICOMT::AutoLock t_lock(ps_lock_);
   CORBA::ULong len = property_modes.length();
   assert (len > 0);
 
@@ -988,7 +1029,7 @@ PropertySetDef_impl::set_property_modes (const PropertyService::PropertyModes
   }
 
   if (mul_exc.exceptions.length () != 0) {
-    mico_throw (PropertyService::MultipleExceptions (mul_exc));
+    mico_throw (mul_exc);
   }
 }
 
@@ -999,6 +1040,7 @@ PropertySetDef_impl::set_property_modes (const PropertyService::PropertyModes
 /*****************************************************************************/
 
 PropertyNamesIterator_impl::PropertyNamesIterator_impl ()
+    : pni_lock_(FALSE, MICOMT::Mutex::Recursive)
 {
   mp_property_set = 0;
   m_index = 0;
@@ -1007,6 +1049,7 @@ PropertyNamesIterator_impl::PropertyNamesIterator_impl ()
 PropertyNamesIterator_impl::PropertyNamesIterator_impl (PropertySet_impl
                                                           *pro_set,
                                                         CORBA::ULong idx)
+    : pni_lock_(FALSE, MICOMT::Mutex::Recursive)
 {
   mp_property_set = pro_set;
   m_index = idx;
@@ -1019,14 +1062,16 @@ PropertyNamesIterator_impl::~PropertyNamesIterator_impl ()
 void
 PropertyNamesIterator_impl::reset ()
 {
+  MICOMT::AutoLock t_lock(pni_lock_);
   assert (mp_property_set != 0);
 
   m_index = 0; 
 }
 
 CORBA::Boolean
-PropertyNamesIterator_impl::next_one (char *&property_name)
+PropertyNamesIterator_impl::next_one (CORBA::String_out property_name)
 {
+  MICOMT::AutoLock t_lock(pni_lock_);
   assert (mp_property_set != 0);
 
   if (m_index >= mp_property_set->get_number_of_properties ())
@@ -1045,8 +1090,9 @@ PropertyNamesIterator_impl::next_one (char *&property_name)
 
 CORBA::Boolean
 PropertyNamesIterator_impl::next_n (CORBA::ULong how_many, 
-                                PropertyService::PropertyNames *&property_names)
+                                PropertyService::PropertyNames_out property_names)
 {
+  MICOMT::AutoLock t_lock(pni_lock_);
   assert (mp_property_set != 0);
 
   CORBA::ULong len = mp_property_set->get_number_of_properties ();
@@ -1081,9 +1127,13 @@ PropertyNamesIterator_impl::next_n (CORBA::ULong how_many,
 void
 PropertyNamesIterator_impl::destroy ()
 {
+  MICOMT::AutoLock t_lock(pni_lock_);
   mp_property_set = 0;
   m_index = 0;
-  CORBA::release (this);
+
+  PortableServer::ObjectId_var oid = _default_POA ()->servant_to_id (this);
+  _default_POA ()->deactivate_object (*oid);
+  this->_remove_ref();
 }
 
 
@@ -1094,6 +1144,7 @@ PropertyNamesIterator_impl::destroy ()
 /*****************************************************************************/
 
 PropertiesIterator_impl::PropertiesIterator_impl ()
+    : pi_lock_(FALSE, MICOMT::Mutex::Recursive)
 {
   mp_property_set = 0;
   m_index = 0;
@@ -1101,6 +1152,7 @@ PropertiesIterator_impl::PropertiesIterator_impl ()
 
 PropertiesIterator_impl::PropertiesIterator_impl (PropertySet_impl *pro_set,
                                                   CORBA::ULong idx)
+    : pi_lock_(FALSE, MICOMT::Mutex::Recursive)
 {
   mp_property_set = pro_set;
   m_index = idx;
@@ -1113,14 +1165,16 @@ PropertiesIterator_impl::~PropertiesIterator_impl ()
 void
 PropertiesIterator_impl::reset()
 {
+  MICOMT::AutoLock t_lock(pi_lock_);
   assert (mp_property_set != 0);
 
   m_index = 0;
 }
 
 CORBA::Boolean
-PropertiesIterator_impl::next_one (PropertyService::Property *&aproperty)
+PropertiesIterator_impl::next_one (PropertyService::Property_out aproperty)
 {
+  MICOMT::AutoLock t_lock(pi_lock_);
   assert (mp_property_set != 0);
 
   if (m_index >= mp_property_set->get_number_of_properties ())
@@ -1145,8 +1199,9 @@ PropertiesIterator_impl::next_one (PropertyService::Property *&aproperty)
 
 CORBA::Boolean
 PropertiesIterator_impl::next_n (CORBA::ULong how_many,
-                                 PropertyService::Properties *&nproperties)
+                                 PropertyService::Properties_out nproperties)
 {
+  MICOMT::AutoLock t_lock(pi_lock_);
   assert (mp_property_set != 0);
 
   CORBA::ULong len = mp_property_set->get_number_of_properties ();
@@ -1184,8 +1239,12 @@ PropertiesIterator_impl::next_n (CORBA::ULong how_many,
 void
 PropertiesIterator_impl::destroy ()
 {
+  MICOMT::AutoLock t_lock(pi_lock_);
   mp_property_set = 0;  
   m_index = 0;
-  CORBA::release (this);
+
+  PortableServer::ObjectId_var oid = _default_POA ()->servant_to_id (this);
+  _default_POA ()->deactivate_object (*oid);
+  this->_remove_ref();
 }
 
