@@ -303,93 +303,6 @@ void CodeGenCPPSkel::emit_skel()
       get_dispatcher_list( in, operations, attributes );
     }
 
-    if (_params.boa && !CORBA::is_nil (in) &&
-	in->def_kind() == CORBA::dk_Interface) {
-      o << endl;
-      o << absClassName << _SKEL << "::" << className << _SKEL;
-      o << "( const CORBA::BOA::ReferenceData &_id )" << endl;
-      o << BL_OPEN;
-      o << "CORBA::ImplementationDef_var _impl =" << indent << endl;
-      o << "_find_impl( \"" << in_id << "\", \"";
-      o << className << "\" );" << exdent << endl;
-      o << "_create_ref( _id," << indent << endl;
-      o << "0," << endl;
-      o << "_impl," << endl;
-      o << "\"" << in_id << "\" );" << exdent << endl;
-      o << "register_dispatcher( new ";
-      o << "StaticInterfaceDispatcherWrapper< ";
-      o << className << _SKEL;
-      o << ">( this ) );" << endl;
-      o << BL_CLOSE << endl;
-      
-      o << absClassName << _SKEL << "::" << className << _SKEL;
-      o << "( CORBA::Object_ptr _obj )" << endl;
-      o << BL_OPEN;
-      o << "CORBA::ImplementationDef_var _impl =" << indent << endl;
-      o << "_find_impl( \"" << in->id() << "\", \"";
-      o << className << "\" );" << exdent << endl;
-      o << "assert( !CORBA::is_nil( _impl ) );" << endl;
-      o << "_restore_ref( _obj," << indent << endl;
-      o << "CORBA::BOA::ReferenceData()," << endl;
-      o << "0," << endl;
-      o << "_impl );" << exdent << endl;
-      o << "register_dispatcher( new ";
-      o << "StaticInterfaceDispatcherWrapper< ";
-      o << className << _SKEL;
-      o << ">( this ) );" << endl;
-      o << BL_CLOSE << endl;
-
-      // dtor
-      o << absClassName << _SKEL;
-      o << "::~" << className << _SKEL << "()" << endl;
-      o << "{" << endl;
-      o << "}" << endl << endl;
-
-      // Generate method dispatcher for SII
-      o << "bool " << absClassName << _SKEL << "::dispatch( ";
-      o << "CORBA::StaticServerRequest_ptr ";
-
-      if (attributes.length() + operations.length() == 0) {
-	o << "/*__req*/";
-      } else {
-	o << "__req";
-      }
-      o << ", CORBA::Environment & /*_env*/ )" << endl << BL_OPEN;
-
-      if( !_params.no_exceptions &&
-	  attributes.length() + operations.length() > 0 ) {
-	o << "#ifdef HAVE_EXCEPTIONS" << endl;
-	o << "try {" << endl;
-	o << "#endif" << indent << endl;
-      }
-
-      emit_dispatcher (operations, attributes);
-      
-      if( !_params.no_exceptions &&
-	  attributes.length() + operations.length() > 0 ) {
-	o << exdent << "#ifdef HAVE_EXCEPTIONS" << endl;
-	o << "} catch( CORBA::SystemException_catch &_ex ) " << BL_OPEN;
-	o << "__req->set_exception( _ex->_clone() );" << endl;
-	o << "__req->write_results();" << endl;
-	o << "return true;" << exdent << endl;
-	o << "} catch( ... ) " << BL_OPEN;
-	o << "assert( 0 );" << endl;
-	o << "return true;" << endl;
-	o << BL_CLOSE;
-	o << "#endif" << endl;
-      }
-
-      o << "return false;" << endl;
-      o << BL_CLOSE << endl;
-      
-      // _this method
-      o << absClassName << "_ptr " << absClassName << _SKEL;
-      o << "::_this()" << endl;
-      o << BL_OPEN;
-      o << "return " << absClassName << "::_duplicate( this );" << endl;
-      o << BL_CLOSE << endl;
-    }
-      
     /*
      * PortableServer Skeleton class
      */
@@ -452,6 +365,11 @@ void CodeGenCPPSkel::emit_skel()
 	  o << "return TRUE;" << endl;
 	  o << BL_CLOSE;
 	}
+	if (_params.reflection) {
+	  o << "if (POA_Reflection::IFRProvider::_is_a (repoid)) " << BL_OPEN;
+	  o << "return TRUE;" << endl;
+	  o << BL_CLOSE;
+	}
       }
       else {
 	/*
@@ -515,6 +433,125 @@ void CodeGenCPPSkel::emit_skel()
       o << "return CORBA::string_dup (\"" << in_id << "\");" << endl;
       o << BL_CLOSE << endl;
 
+      if (_params.reflection) {
+	try {
+	  CORBA::ExtInterfaceDef_var eid = CORBA::ExtInterfaceDef::_narrow (in);
+	  if (CORBA::is_nil (eid)) {
+	    throw string ("oops");
+	  }
+
+	  CORBA::InterfaceAttrExtension::ExtFullInterfaceDescription_var efid =
+	    eid->describe_ext_interface ();
+	  CORBA::Any aefid;
+	  aefid <<= efid.in();
+
+	  CORBA::ORB_var orb = CORBA::ORB_instance ("mico-local-orb");
+	  if (CORBA::is_nil (orb)) {
+	    throw string ("oops");
+	  }
+
+	  CORBA::Object_var oc = orb->resolve_initial_references ("CodecFactory");
+	  IOP::CodecFactory_var cf = IOP::CodecFactory::_narrow (oc);
+	  if (CORBA::is_nil (cf)) {
+	    throw string ("oops");
+	  }
+
+	  IOP::Encoding enc;
+	  enc.format = IOP::ENCODING_CDR_ENCAPS;
+	  enc.major_version = 1;
+	  enc.minor_version = 2;
+	  IOP::Codec_var codec = cf->create_codec (enc);
+
+	  CORBA::OctetSeq_var data = codec->encode_value (aefid);
+
+	  o << "CORBA::Any *" << endl;
+	  o << "POA_" << absClassName << "::omg_get_ifr_metadata ()" << endl;
+	  o << BL_OPEN;
+
+	  const CORBA::ULong maxOctetsPerArray = 16384;
+	  CORBA::ULong numArrays = (data->length() + maxOctetsPerArray - 1) / maxOctetsPerArray;
+	  CORBA::ULong lastDimOctets = data->length() - (numArrays - 1) * maxOctetsPerArray;
+
+	  for (CORBA::ULong aidx=0; aidx<numArrays; aidx++) {
+	    CORBA::ULong dimOctets = (aidx+1<numArrays) ? maxOctetsPerArray : lastDimOctets;
+
+	    o << "static CORBA::Octet rawdata_" << aidx << "[" << dimOctets << "] = " << BL_OPEN;
+
+	    for (CORBA::ULong didx=0; didx<dimOctets; didx++) {
+	      o << (int) data[didx+aidx*maxOctetsPerArray];
+
+	      if (didx+1 < dimOctets) {
+		o << ",";
+	      }
+	      if (didx+1 == dimOctets || ((didx+1) % 16) == 0) {
+		o << endl;
+	      }
+	      else {
+		o << " ";
+	      }
+	    }
+	    o << BL_CLOSE_SEMI;
+	  }
+	  o << endl;
+
+	  o << "static CORBA::OctetSeq data;" << endl;
+	  o << endl;
+	  o << "if (!data.length()) " << BL_OPEN;
+	  o << "data.length (" << data->length() << ");" << endl;
+	  o << "CORBA::Octet * dptr = data.get_buffer ();" << endl;
+	  for (CORBA::ULong aidx2=0; aidx2<numArrays; aidx2++) {
+	    CORBA::ULong dimOctets = (aidx2+1<numArrays) ? maxOctetsPerArray : lastDimOctets;
+	    o << "memcpy (dptr";
+	    if (aidx2) {
+	      o << "+" << aidx2*maxOctetsPerArray;
+	    }
+	    o << ", rawdata_" << aidx2 << ", " << dimOctets << ");" << endl;
+	  }
+	  o << BL_CLOSE << endl;
+	  o << "try " << BL_OPEN;
+	  o << "CORBA::ORB_var orb = CORBA::ORB_instance (\"mico-local-orb\");" << endl;
+	  o << "if (CORBA::is_nil (orb)) " << BL_OPEN;
+	  o << "throw Reflection::FormatNotSupported ();" << endl;
+	  o << BL_CLOSE;
+	  o << "CORBA::Object_var oc = orb->resolve_initial_references (\"CodecFactory\");" << endl;
+	  o << "IOP::CodecFactory_var cf = IOP::CodecFactory::_narrow (oc);" << endl;
+	  o << "if (CORBA::is_nil (cf)) " << BL_OPEN;
+	  o << "throw Reflection::FormatNotSupported ();" << endl;
+	  o << BL_CLOSE;
+	  o << "IOP::Encoding enc;" << endl;
+	  o << "enc.format = IOP::ENCODING_CDR_ENCAPS;" << endl;
+	  o << "enc.major_version = 1;" << endl;
+	  o << "enc.minor_version = 2;" << endl;
+	  o << "IOP::Codec_var codec = cf->create_codec (enc);" << endl;
+	  o << "return codec->decode_value (data, CORBA::InterfaceAttrExtension::_tc_ExtFullInterfaceDescription);" << endl;
+	  o << BL_CLOSE;
+	  o << "catch (const Reflection::FormatNotSupported &) " << BL_OPEN;
+	  o << "throw;" << endl;
+	  o << BL_CLOSE;
+	  o << "catch (...) " << BL_OPEN;
+	  o << "throw Reflection::FormatNotSupported ();" << endl;
+	  o << BL_CLOSE;
+	  o << BL_CLOSE << endl;
+	}
+	catch (...) {
+	  /*
+	   * Couldn't get or encode the ExtFullInterfaceDescription for this interface.
+	   */
+
+	  o << "CORBA::Any *" << endl;
+	  o << "POA_" << absClassName << "::omg_get_ifr_metadata ()" << endl;
+	  o << BL_OPEN;
+	  o << "throw Reflection::FormatNotSupported ();" << endl;
+	  o << BL_CLOSE << endl;
+	}
+
+	o << "char *" << endl;
+	o << "POA_" << absClassName << "::omg_get_xml_metadata ()" << endl;
+	o << BL_OPEN;
+	o << "throw Reflection::FormatNotSupported ();" << endl;
+	o << BL_CLOSE << endl;
+      }
+
       /*
        * _narrow_helper() and _narrow() moved to stub
        */
@@ -588,6 +625,11 @@ void CodeGenCPPSkel::emit_skel()
 	  o << "return true;" << endl;
 	  o << BL_CLOSE << endl;
 	}
+	if (_params.reflection) {
+	  o << "if (POA_Reflection::IFRProvider::dispatch (__req)) " << BL_OPEN;
+	  o << "return true;" << endl;
+	  o << BL_CLOSE << endl;
+	}
       }
       else {
 	string base_name;
