@@ -20,10 +20,23 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "idl_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #include <CORBA.h>
 #include "parser.h"
 #include <mico/template_impl.h>
 
+#endif // FAST_PCH
+
+
+using namespace std;
 
 extern FILE* yyin;
 extern char* yytext;
@@ -37,6 +50,7 @@ Parser::Parser( FILE *inp_file, const char *filename )
 {
   idl_lexFile = toplevelFile = (char *) filename;
   yyin = inp_file;
+  rootNode = 0;
 }
 
 Parser::~Parser ()
@@ -46,10 +60,12 @@ Parser::~Parser ()
 
 void Parser::parse()
 {
+  Parser * oldParser = theParser;
   theParser = this;
   idl_line_no = 0;
   /* yydebug = 1; */
   yyparse();
+  theParser = oldParser;
 }
 
 
@@ -64,18 +80,6 @@ ParseNode *Parser::getRootNode()
   return rootNode;
 }
 
-void Parser::setImports( ParseNode *node )
-{
-  imports = node;
-}
-
-
-ParseNode *Parser::getImports()
-{
-  return imports;
-}
-
-
 void Parser::parse_error( char *file, char *err, int line )
 {
   string msg;
