@@ -20,12 +20,25 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "idl_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #include <CORBA.h>
 #include <string>
 #include <stdlib.h>
 #include "prepro.h"
 #include <mico/template_impl.h>
 
+#endif // FAST_PCH
+
+
+using namespace std;
 
 char  *toplevelFile = 0;
 char  *idl_lexFile = 0;
