@@ -1,7 +1,7 @@
 // -*- c++ -*-
 //
 //  MICO PI --- an Open Source Portable Interceptor implementation
-//  Copyright (C) 2001, 2002, 2003 ObjectSecurity Ltd.
+//  Copyright (C) 2001, 2002, 2003, 2004, 2005 ObjectSecurity Ltd.
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Library General Public
@@ -31,9 +31,19 @@ namespace PortableInterceptor
     extern void
     register_orb_initializer(PortableInterceptor::ORBInitializer_ptr init);
 
+    // extension
     extern void
     destroy_all_interceptors();
 
+    extern void
+    backup_initializers();
+
+    extern void
+    restore_initializers();
+
+    extern void
+    commit_initialized_interceptors();
+
     typedef void (ClientRequestInterceptor::*ClientMethod)
 	(PortableInterceptor::ClientRequestInfo_ptr ri);
 
@@ -505,7 +515,7 @@ namespace PInterceptor
 	virtual public CORBA::LocalObject
     {
     private:
-	CORBA::ORB_ptr orb_;
+	CORBA::ORB_var orb_;
 	std::string orb_id_;
 	CORBA::StringSeq args_;
     public:
@@ -572,17 +582,22 @@ namespace PInterceptor
     };
 
     struct PI {
-	static std::list<PortableInterceptor::ORBInitializer_ptr> S_initializers_;
+	static std::vector<PortableInterceptor::ORBInitializer_var> S_initializers_;
+	static std::vector<PortableInterceptor::ORBInitializer_var> S_initializers_backup_;
 	static PortableInterceptor::SlotId S_max_slot_id_;
-	typedef std::list<PortableInterceptor::IORInterceptor_ptr> IORList;
+	typedef std::list<PortableInterceptor::IORInterceptor_var> IORList;
 	typedef std::map<CORBA::PolicyType, PortableInterceptor::PolicyFactory_ptr,
 			 std::less<CORBA::PolicyType> > PolicyFactoryMap;
 	static PolicyFactoryMap S_pfmap_;
-	
+
 	static std::vector<PortableInterceptor::ClientRequestInterceptor_ptr>
 	S_client_req_int_;
+	static std::vector<PortableInterceptor::ClientRequestInterceptor_ptr>
+	S_client_req_int_prepared_;
 	static std::vector<PortableInterceptor::ServerRequestInterceptor_ptr>
 	S_server_req_int_;
+	static std::vector<PortableInterceptor::ServerRequestInterceptor_ptr>
+	S_server_req_int_prepared_;
 
 	typedef FlowStack<PortableInterceptor::ClientRequestInterceptor_ptr>
 	ClientFlowStack;
@@ -595,6 +610,7 @@ namespace PInterceptor
 	static MICOMT::Mutex S_server_lock_;
 
 	static IORList S_ior_interceptors_;
+        static IORList S_ior_interceptors_prepared_;
 	
 	static void
 	_init ();
