@@ -1,7 +1,7 @@
 // -*- c++ -*-
 /*
  *  MICO --- an Open Source CORBA implementation
- *  Copyright (c) 1997-2001 by The Mico Team
+ *  Copyright (c) 1997-2005 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -96,6 +96,7 @@ public:
 	CORBA::Octet *, CORBA::ULong, const CORBA::MultiComponent &,
         CORBA::UShort version = 0x0100) const;
     CORBA::Boolean is_local () const;
+    CORBA::Boolean is_here () const;
     
     CORBA::Address *clone () const;
     
@@ -116,7 +117,9 @@ public:
 };
 
 
-class SSLProfile : public CORBA::IORProfile {
+class SSLProfile : public CORBA::IORProfile,
+                   public MICO::ProfileIIOPVersionProvider
+{
     CORBA::IORProfile *_prof;
     SSLAddress _myaddr;
 public:
@@ -142,6 +145,8 @@ public:
     CORBA::Long compare (const CORBA::IORProfile &) const;
     CORBA::Boolean operator== (const CORBA::IORProfile &) const;
     CORBA::Boolean operator< (const CORBA::IORProfile &) const;
+
+    CORBA::UShort iiop_version () const;
 };
 
 #if 0
@@ -216,6 +221,7 @@ class SSLTransport : public MICO::SocketTransport,
 #ifdef HAVE_THREADS
     MICOMT::Mutex _ssl_mutex;
 #endif // HAVE_THREADS
+    CORBA::Boolean _closed;
     std::string _err;
     CORBA::TransportCallback *_rcb, *_wcb;
 #ifndef USE_CSL2
