@@ -20,11 +20,19 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
 
 #include <CORBA-SMALL.h>
 #include <mico/template_impl.h>
 
-
+#endif // FAST_PCH
 
 void
 operator<<=( CORBA::Any &a, const SequenceTmpl<CORBA::Any,MICO_TID_DEF> &s )
