@@ -20,17 +20,30 @@
  *  http://www.mico.org/
  */
 
+#ifdef FAST_PCH
+#include "orb_pch.h"
+#endif // FAST_PCH
+#ifdef __COMO__
+#pragma hdrstop
+#endif // __COMO__
+
+#ifndef FAST_PCH
+
 #define MICO_CONF_IMR
 
 #include <CORBA-SMALL.h>
-#ifndef _WINDOWS
+#ifndef _WIN32
 #include <string.h>
 #endif
-#include <iostream.h>
 #include <mico/impl.h>
 #include <mico/template_impl.h>
 #include <mico/util.h>
 
+#endif // FAST_PCH
+
+
+using namespace std;
+
 /**************************** DataEncoder ******************************/
 
 
@@ -126,38 +139,18 @@ CORBA::DataEncoder::enumeration (ULong val)
     put_ulong (val);
 }
 
-void
-CORBA::DataEncoder::struct_begin ()
-{
-}
-
-void
-CORBA::DataEncoder::struct_end ()
-{
-}
-
 void
 CORBA::DataEncoder::except_begin (const string &repoid)
 {
     put_string (repoid);
 }
 
-void
-CORBA::DataEncoder::except_end ()
-{
-}
-
 void
 CORBA::DataEncoder::seq_begin (ULong l)
 {
     put_ulong (l);
 }
 
-void
-CORBA::DataEncoder::seq_end ()
-{
-}
-
 void
 CORBA::DataEncoder::encaps_begin (EncapsState &state)
 {
@@ -210,26 +203,6 @@ CORBA::DataEncoder::delayed_seq_end (DelayedSeqState &state, ULong l)
     buffer()->wseek_beg (pos);
 }
 
-void
-CORBA::DataEncoder::union_begin ()
-{
-}
-
-void
-CORBA::DataEncoder::union_end ()
-{
-}
-
-void
-CORBA::DataEncoder::arr_begin ()
-{
-}
-
-void
-CORBA::DataEncoder::arr_end ()
-{
-}
-
 void
 CORBA::DataEncoder::value_begin (const string &url,
 				 const vector<string> &repoids,
@@ -502,30 +475,12 @@ CORBA::DataDecoder::enumeration (ULong &val)
     return get_ulong (val);
 }
 
-CORBA::Boolean
-CORBA::DataDecoder::struct_begin ()
-{
-    return TRUE;
-}
-
-CORBA::Boolean
-CORBA::DataDecoder::struct_end ()
-{
-    return TRUE;
-}
-
 CORBA::Boolean
 CORBA::DataDecoder::except_begin (string &repoid)
 {
     return get_string_stl (repoid);
 }
 
-CORBA::Boolean
-CORBA::DataDecoder::except_end ()
-{
-    return TRUE;
-}
-
 CORBA::Boolean
 CORBA::DataDecoder::seq_begin (ULong &l)
 {
@@ -551,12 +506,6 @@ CORBA::DataDecoder::seq_begin (ULong &l)
   return TRUE;
 }
 
-CORBA::Boolean
-CORBA::DataDecoder::seq_end ()
-{
-    return TRUE;
-}
-
 CORBA::Boolean
 CORBA::DataDecoder::encaps_begin (EncapsState &state, ULong &len)
 {
@@ -598,37 +547,13 @@ CORBA::DataDecoder::encaps_end (EncapsState &state)
     return TRUE;
 }
 
-CORBA::Boolean
-CORBA::DataDecoder::union_begin ()
-{
-    return TRUE;
-}
-
-CORBA::Boolean
-CORBA::DataDecoder::union_end ()
-{
-    return TRUE;
-}
-
-CORBA::Boolean
-CORBA::DataDecoder::arr_begin ()
-{
-    return TRUE;
-}
-
-CORBA::Boolean
-CORBA::DataDecoder::arr_end ()
-{
-    return TRUE;
-}
-
-
 CORBA::Boolean
 CORBA::DataDecoder::get_indirect_string (string &s)
 {
-    Long tag, rpos = buf->rpos();
+    Long rpos = buf->rpos();
+    ULong tag = 0;
 
-    if (!get_long (tag))
+    if (!get_ulong (tag))
 	return FALSE;
     if (tag == 0xffffffff) {
 	// indirection
@@ -655,9 +580,10 @@ CORBA::DataDecoder::get_indirect_string (string &s)
 CORBA::Boolean
 CORBA::DataDecoder::get_indirect_string_seq (vector<string> &v)
 {
-    Long tag, rpos = buf->rpos();
+    Long rpos = buf->rpos();
+    ULong tag = 0;
 
-    if (!get_long (tag))
+    if (!get_ulong (tag))
 	return FALSE;
     if (tag == 0xffffffff) {
 	// indirection
@@ -701,6 +627,45 @@ CORBA::DataDecoder::get_indirect_string_seq (vector<string> &v)
     return TRUE;
 }
 
+CORBA::Boolean
+CORBA::DataDecoder::begin_chunk ()
+{
+  assert (vstate);
+  assert (vstate->s.chunking);
+
+  CORBA::Long length_of_chunk;
+
+  if (!get_long (length_of_chunk)) {
+    return FALSE;
+  }
+
+  if (length_of_chunk < 0 || (CORBA::ULong)length_of_chunk > buf->length()) {
+    return FALSE;
+  }
+
+  vstate->s.end_of_chunk = buf->rpos () + length_of_chunk;
+
+  return TRUE;
+}
+
+CORBA::Boolean
+CORBA::DataDecoder::end_chunk ()
+{
+  assert (vstate);
+  assert (vstate->s.chunking);
+
+  if (vstate->s.end_of_chunk == -1) {
+    return TRUE;
+  }
+
+  if (!buf->rseek_beg (vstate->s.end_of_chunk)) {
+    return FALSE;
+  }
+
+  vstate->s.end_of_chunk = -1;
+  return TRUE;
+}
+
 CORBA::Boolean
 CORBA::DataDecoder::value_begin (string &url,
 				 vector<string> &repoids,
@@ -718,19 +683,19 @@ CORBA::DataDecoder::value_begin (string &url,
 				 Boolean &is_ref,
 				 ValueState &state)
 {
-    Long value_tag;
+    ULong value_tag;
 
-    if (!get_long (value_tag))
+    if (!get_ulong (value_tag))
 	return FALSE;
 
     if (value_tag == 0xffffffff) {
 	// indirection
-	Long indir, rpos = buf->rpos();
+	Long indir;
 	if (!get_long (indir))
 	    return FALSE;
 
 	// +1 to avoid value_id 0
-	value_id = rpos + indir + 1;
+	value_id = buf->rpos() - sizeof(Long) + indir + 1;
 
 	MapIdState::iterator i = state.skipped.find (value_id);
 	if (i != state.skipped.end() && (*i).second.saved_pos < 0) {
@@ -759,6 +724,7 @@ CORBA::DataDecoder::value_begin (string &url,
     }
     if (!(0x7fffff00 <= value_tag && value_tag <= 0x7fffffff)) {
       if (MICO::Logger::IsLogged (MICO::Logger::Info)) {
+	MICOMT::AutoDebugLock __lock;
 	MICO::Logger::Stream (MICO::Logger::Info)
 	  << "Info: DataDecoder::value_begin(): bad tag: "
 	  << "0x" << hex << value_tag << endl;
@@ -790,40 +756,17 @@ CORBA::DataDecoder::value_begin (string &url,
 	    return FALSE;
     }
 
-    state.s.nesting_level += 1;
-
     if (value_tag & 0x08) {
-	// chunk flag
-	if (state.s.chunk_level > state.s.nesting_level)
-	    state.s.chunk_level = state.s.nesting_level;
+      state.s.chunking = 1;
     }
 
-    if (state.s.chunk_level <= state.s.nesting_level) {
-	// chunked
-	state.s.data_nesting_level += 1;
-
-	// try to open next chunk
-	Long tag, rpos = buf->rpos();
-
-	if (!get_long (tag))
-	    return FALSE;
-
-	if (0 < tag && tag < 0x7fffff00) {
-	    // ok, its a chunk
-	    rpos = buf->rpos();
-	    // make sure next tag is no block size tag, which
-	    // is currently not supported, because then we
-	    // had to check for end of chunk after each get_*
-	    // operation ...
-	    if (!buf->rseek_rel (tag))
-		return FALSE;
-	    if (!get_long (tag))
-		return FALSE;
-	    if (0 < tag && tag < 0x7fffff00)
-	      return FALSE;
-	}
-	buf->rseek_beg (rpos);
+    if (state.s.chunking) {
+      state.s.nesting_level++;
+      if (!begin_chunk()) {
+	return FALSE;
+      }
     }
+
     return TRUE;
 }
 
@@ -840,8 +783,6 @@ CORBA::DataDecoder::value_end (Long finished_value_id, ValueState &state)
     // must not be called if value_begin() returned an indirection or nil,
     // i.e. if is_ref == TRUE!
 
-    assert (state.s.nesting_level > 0);
-
     MapIdState::iterator i = state.skipped.find (finished_value_id);
     if (i != state.skipped.end() && (*i).second.saved_pos >= 0) {
 	// we just finished decoding a value that had been
@@ -855,116 +796,77 @@ CORBA::DataDecoder::value_end (Long finished_value_id, ValueState &state)
 	return TRUE;
     }
 
-    if (!(state.s.chunk_level <= state.s.nesting_level)) {
-	// not chunked
-	state.s.nesting_level -= 1;
-	return TRUE;
-    }
+    if (state.s.chunking) {
+      if (!end_chunk()) {
+	return FALSE;
+      }
 
-    if (state.s.data_nesting_level < state.s.nesting_level) {
-	// we already read the end tag ...
-	state.s.nesting_level -= 1;
-	return TRUE;
-    }
+      Long level = state.s.nesting_level + 1;
+      Long tag, pos = 0;
 
-    // skip until we read our end tag
-    while (state.s.data_nesting_level >= state.s.nesting_level) {
-	Long tag, rpos = buf->rpos();
-	if (!get_long (tag))
-	    return FALSE;
-	if (0 < tag && tag < 0x7fffff00) {
-	    // chunk
-	    if (!buf->rseek_rel (tag))
-		return FALSE;
-	} else if (tag < 0) {
-	    // end tag
-	    if (state.s.data_nesting_level > -tag-1) {
-		state.s.data_nesting_level = -tag-1;
-	    } else {
-	      if (MICO::Logger::IsLogged (MICO::Logger::Info)) {
-		MICO::Logger::Stream (MICO::Logger::Info)
-		    << "Info: DataDecoder::value_end(): bad end tag: "
-		    << tag << " (ignored)" << endl;
-	      }
-	    }
-	} else {
-	    // value or value_ref or nil
-	    buf->rseek_beg (rpos);
+      while (level > state.s.nesting_level) {
+	pos = buf->rpos();
 
-	    string url;
-	    vector<string> repoids;
-	    Long value_id;
-	    Boolean is_ref;
-	    ValueState1 saved_state = state.s;
-
-	    if (!value_begin (url, repoids, value_id, is_ref, state))
-		return FALSE;
-	    if (!is_ref) {
-		if (!value_end (value_id, state))
-		    return FALSE;
-		// later we may come across indirections that refer
-		// to the value we just skipped. when we come across
-		// the first such reference we have to treat the
-		// indirection as if is was the value itself. therefore
-		// we have to save the state ...
-		saved_state.saved_pos = -1;
-		state.skipped[value_id] = saved_state;
-	    }
+	if (!get_long (tag)) {
+	  return FALSE;
 	}
-    }
-    state.s.nesting_level -= 1;
-
-    while (state.s.data_nesting_level > 0 &&
-	   state.s.chunk_level <= state.s.data_nesting_level) {
-	// still chunked, eat next end tag
-	Long tag, rpos = buf->rpos();
-	if (!get_long (tag))
+	if (tag > 0 && tag < 0x7fffff00) {
+	  /*
+	   * Another chunk follows with truncated data. Ignore.
+	   */
+	  buf->rseek_beg (pos);
+	  if (!begin_chunk() || !end_chunk()) {
 	    return FALSE;
-
-	if (tag < 0) {
-	    // end tag
-	    if (state.s.data_nesting_level > -tag-1) {
-		state.s.data_nesting_level = -tag-1;
-	    } else {
-	      if (MICO::Logger::IsLogged (MICO::Logger::Info)) {
-		MICO::Logger::Stream (MICO::Logger::Info)
-		  << "Info: DataDecoder::value_end(): bad end tag: "
-		  << tag << " (ignored)" << endl;
-	      }
-	    }
-	} else {
-	    // some other tag
-	    buf->rseek_beg (rpos);
-	    break;
+	  }
 	}
-    }
-    if (state.s.data_nesting_level > 0 &&
-	state.s.chunk_level <= state.s.data_nesting_level) {
-	// still chunked, try to open next chunk
-	Long tag, rpos = buf->rpos();
+	else if (tag < 0) {
+	  /*
+	   * end tag
+	   */
+	  level = -tag;
+	}
+	else {
+	  /*
+	   * nested value follows
+	   */
 
-	if (!get_long (tag))
+	  buf->rseek_beg (pos);
+	  
+	  string url;
+	  vector<string> repoids;
+	  Long value_id;
+	  Boolean is_ref;
+	  ValueState1 saved_state = state.s;
+	  
+	  if (!value_begin (url, repoids, value_id, is_ref, state))
 	    return FALSE;
-
-	if (0 < tag && tag < 0x7fffff00) {
-	    // ok, its a chunk
-	    rpos = buf->rpos();
-	    // make sure next tag is no block size tag, which
-	    // is currently not supported, because then we
-	    // had to check for end of chunk after each get_*
-	    // operation ...
-	    if (!buf->rseek_rel (tag))
-		return FALSE;
-	    if (!get_long (tag))
-		return FALSE;
-	    if (0 < tag && tag < 0x7fffff00)
+	  if (!is_ref) {
+	    if (!value_end (value_id, state))
 	      return FALSE;
+	    // later we may come across indirections that refer
+	    // to the value we just skipped. when we come across
+	    // the first such reference we have to treat the
+	    // indirection as if is was the value itself. therefore
+	    // we have to save the state ...
+	    saved_state.saved_pos = -1;
+	    state.skipped[value_id] = saved_state;
+	  }
 	}
-	buf->rseek_beg (rpos);
-    } else {
-	// now follows unchunked data (or no data at all)
-	state.s.chunk_level = 0x7fffffff;
+      }
+
+      if (level < state.s.nesting_level) {
+	/*
+	 * seek back before end tag, so that outer end_value()
+	 * can read it
+	 */
+	buf->rseek_beg (pos);
+      }
+
+      if (--state.s.nesting_level == 0) {
+	state.s.chunking = FALSE;
+      }
     }
+
     return TRUE;
 }
 
@@ -1040,7 +942,9 @@ static inline void swap8 (void *d, const void *s)
     ((CORBA::Octet *)d)[7] = ((CORBA::Octet *)s)[0];
 }
 
-static inline void swap16 (void *d, const void *s)
+
+// OpenBSD defines macro swap16
+static inline void swap_16 (void *d, const void *s)
 {
     ((CORBA::Octet *)d)[0]  = ((CORBA::Octet *)s)[15];
     ((CORBA::Octet *)d)[1]  = ((CORBA::Octet *)s)[14];
@@ -1092,6 +996,7 @@ MICO::CDREncoder::~CDREncoder ()
 void
 MICO::CDREncoder::put_short (CORBA::Short s)
 {
+  buf->walign (2);
     if (mach_bo == data_bo) {
 	buf->put2 (&s);
     } else {
@@ -1104,6 +1009,7 @@ MICO::CDREncoder::put_short (CORBA::Short s)
 void
 MICO::CDREncoder::put_ushort (CORBA::UShort us)
 {
+  buf->walign (2);
     if (mach_bo == data_bo) {
 	buf->put2 (&us);
     } else {
@@ -1116,6 +1022,7 @@ MICO::CDREncoder::put_ushort (CORBA::UShort us)
 void
 MICO::CDREncoder::put_long (CORBA::Long l)
 {
+  buf->walign (4);
     if (mach_bo == data_bo) {
 	buf->put4 (&l);
     } else {
@@ -1128,6 +1035,7 @@ MICO::CDREncoder::put_long (CORBA::Long l)
 void
 MICO::CDREncoder::put_longlong (CORBA::LongLong l)
 {
+  buf->walign (8);
     if (mach_bo == data_bo) {
 	buf->put8 (&l);
     } else {
@@ -1140,6 +1048,7 @@ MICO::CDREncoder::put_longlong (CORBA::LongLong l)
 void
 MICO::CDREncoder::put_ulong (CORBA::ULong ul)
 {
+  buf->walign (4);
     if (mach_bo == data_bo) {
 	buf->put4 (&ul);
     } else {
@@ -1152,6 +1061,7 @@ MICO::CDREncoder::put_ulong (CORBA::ULong ul)
 void
 MICO::CDREncoder::put_ulonglong (CORBA::ULongLong ul)
 {
+  buf->walign (8);
     if (mach_bo == data_bo) {
 	buf->put8 (&ul);
     } else {
@@ -1164,6 +1074,7 @@ MICO::CDREncoder::put_ulonglong (CORBA::ULongLong ul)
 void
 MICO::CDREncoder::put_float (CORBA::Float f)
 {
+  buf->walign (4);
 #ifdef HAVE_IEEE_FP
     if (mach_bo == data_bo) {
 	buf->put4 (&f);
@@ -1188,6 +1099,7 @@ MICO::CDREncoder::put_float (CORBA::Float f)
 void
 MICO::CDREncoder::put_double (CORBA::Double d)
 {
+  buf->walign (8);
 #ifdef HAVE_IEEE_FP
     if (mach_bo == data_bo) {
 	buf->put8 (&d);
@@ -1212,12 +1124,13 @@ MICO::CDREncoder::put_double (CORBA::Double d)
 void
 MICO::CDREncoder::put_longdouble (CORBA::LongDouble d)
 {
+  buf->walign (8);
 #if defined(HAVE_IEEE_FP) && SIZEOF_LONG_DOUBLE == 16
     if (mach_bo == data_bo) {
 	buf->put16 (&d);
     } else {
 	CORBA::LongDouble d2;
-	swap16 (&d2, &d);
+	swap_16 (&d2, &d);
 	buf->put16 (&d2);
     }
 #else
@@ -1227,7 +1140,7 @@ MICO::CDREncoder::put_longdouble (CORBA::LongDouble d)
 	buf->put16 (b);
     } else {
 	CORBA::Octet b2[16];
-	swap16 (b2, b);
+	swap_16 (b2, b);
 	buf->put16 (b2);
     }
 #endif
@@ -1441,7 +1354,9 @@ MICO::CDREncoder::put_wstring (const wchar_t *s)
     if (!conv) {
       CORBA::ULong len = xwcslen (s) + 1;
       put_ulong (len);
-      buf->put (s, len * sizeof (wchar_t));
+      while (len--) {
+	put_ushort ((CORBA::UShort) *s++);
+      }
       return;
     }
 
@@ -1565,6 +1480,10 @@ MICO::CDRDecoder::~CDRDecoder ()
 CORBA::Boolean
 MICO::CDRDecoder::get_short (CORBA::Short &s)
 {
+  if (!buf->ralign (2) || !check_chunk ()) {
+    return FALSE;
+  }
+
     if (data_bo == mach_bo)
 	return buf->get2 (&s);
 
@@ -1578,6 +1497,10 @@ MICO::CDRDecoder::get_short (CORBA::Short &s)
 CORBA::Boolean
 MICO::CDRDecoder::get_ushort (CORBA::UShort &us)
 {
+  if (!buf->ralign (2) || !check_chunk ()) {
+    return FALSE;
+  }
+
     if (data_bo == mach_bo)
 	return buf->get2 (&us);
 
@@ -1591,6 +1514,10 @@ MICO::CDRDecoder::get_ushort (CORBA::UShort &us)
 CORBA::Boolean
 MICO::CDRDecoder::get_long (CORBA::Long &l)
 {
+  if (!buf->ralign (4) || !check_chunk ()) {
+    return FALSE;
+  }
+
     if (data_bo == mach_bo)
 	return buf->get4 (&l);
 
@@ -1604,6 +1531,10 @@ MICO::CDRDecoder::get_long (CORBA::Long &l)
 CORBA::Boolean
 MICO::CDRDecoder::get_longlong (CORBA::LongLong &l)
 {
+  if (!buf->ralign (8) || !check_chunk ()) {
+    return FALSE;
+  }
+
     if (data_bo == mach_bo)
 	return buf->get8 (&l);
 
@@ -1617,6 +1548,10 @@ MICO::CDRDecoder::get_longlong (CORBA::LongLong &l)
 CORBA::Boolean
 MICO::CDRDecoder::get_ulong (CORBA::ULong &ul)
 {
+  if (!buf->ralign (4) || !check_chunk ()) {
+    return FALSE;
+  }
+
     if (data_bo == mach_bo)
 	return buf->get4 (&ul);
 
@@ -1630,6 +1565,10 @@ MICO::CDRDecoder::get_ulong (CORBA::ULong &ul)
 CORBA::Boolean
 MICO::CDRDecoder::get_ulonglong (CORBA::ULongLong &ul)
 {
+  if (!buf->ralign (8) || !check_chunk ()) {
+    return FALSE;
+  }
+
     if (data_bo == mach_bo)
 	return buf->get8 (&ul);
 
@@ -1643,6 +1582,10 @@ MICO::CDRDecoder::get_ulonglong (CORBA::ULongLong &ul)
 CORBA::Boolean
 MICO::CDRDecoder::get_float (CORBA::Float &f)
 {
+  if (!buf->ralign (4) || !check_chunk ()) {
+    return FALSE;
+  }
+
 #ifdef HAVE_IEEE_FP
     if (data_bo == mach_bo)
 	return buf->get4 (&f);
@@ -1672,6 +1615,10 @@ MICO::CDRDecoder::get_float (CORBA::Float &f)
 CORBA::Boolean
 MICO::CDRDecoder::get_double (CORBA::Double &d)
 {
+  if (!buf->ralign (8) || !check_chunk ()) {
+    return FALSE;
+  }
+
 #ifdef HAVE_IEEE_FP
     if (data_bo == mach_bo)
 	return buf->get8 (&d);
@@ -1701,6 +1648,10 @@ MICO::CDRDecoder::get_double (CORBA::Double &d)
 CORBA::Boolean
 MICO::CDRDecoder::get_longdouble (CORBA::LongDouble &d)
 {
+  if (!buf->ralign (8) || !check_chunk ()) {
+    return FALSE;
+  }
+
 #if defined(HAVE_IEEE_FP) && SIZEOF_LONG_DOUBLE == 16
     if (data_bo == mach_bo)
 	return buf->get16 (&d);
@@ -1708,7 +1659,7 @@ MICO::CDRDecoder::get_longdouble (CORBA::LongDouble &d)
     CORBA::LongDouble d2;
     if (!buf->get16 (&d2))
 	return FALSE;
-    swap16 (&d, &d2);
+    swap_16 (&d, &d2);
 #else
     CORBA::Octet b[16];
     if (data_bo == mach_bo) {
@@ -1721,7 +1672,7 @@ MICO::CDRDecoder::get_longdouble (CORBA::LongDouble &d)
     CORBA::Octet b2[16];
     if (!buf->get16 (b2))
 	return FALSE;
-    swap16 (&b, &b2);
+    swap_16 (&b, &b2);
     mico_ieee2ldouble (b, d);
 #endif
     return TRUE;
@@ -1768,7 +1719,7 @@ MICO::CDRDecoder::get_boolean (CORBA::Boolean &b)
 CORBA::Boolean
 MICO::CDRDecoder::get_shorts (CORBA::Short *p, CORBA::ULong l)
 {
-    if (!buf->ralign (2))
+    if (!buf->ralign (2) || !check_chunk())
 	return FALSE;
 
     if (data_bo == mach_bo)
@@ -1788,7 +1739,7 @@ MICO::CDRDecoder::get_shorts (CORBA::Short *p, CORBA::ULong l)
 CORBA::Boolean
 MICO::CDRDecoder::get_ushorts (CORBA::UShort *p, CORBA::ULong l)
 {
-    if (!buf->ralign (2))
+    if (!buf->ralign (2) || !check_chunk ())
 	return FALSE;
 
     if (data_bo == mach_bo)
@@ -1808,7 +1759,7 @@ MICO::CDRDecoder::get_ushorts (CORBA::UShort *p, CORBA::ULong l)
 CORBA::Boolean
 MICO::CDRDecoder::get_longs (CORBA::Long *p, CORBA::ULong l)
 {
-    if (!buf->ralign (4))
+    if (!buf->ralign (4) || !check_chunk ())
 	return FALSE;
 
     if (data_bo == mach_bo)
@@ -1828,7 +1779,7 @@ MICO::CDRDecoder::get_longs (CORBA::Long *p, CORBA::ULong l)
 CORBA::Boolean
 MICO::CDRDecoder::get_longlongs (CORBA::LongLong *p, CORBA::ULong l)
 {
-    if (!buf->ralign (8))
+    if (!buf->ralign (8) || !check_chunk ())
 	return FALSE;
 
     if (data_bo == mach_bo)
@@ -1848,7 +1799,7 @@ MICO::CDRDecoder::get_longlongs (CORBA::LongLong *p, CORBA::ULong l)
 CORBA::Boolean
 MICO::CDRDecoder::get_ulongs (CORBA::ULong *p, CORBA::ULong l)
 {
-    if (!buf->ralign (4))
+    if (!buf->ralign (4) || !check_chunk ())
 	return FALSE;
 
     if (data_bo == mach_bo)
@@ -1868,7 +1819,7 @@ MICO::CDRDecoder::get_ulongs (CORBA::ULong *p, CORBA::ULong l)
 CORBA::Boolean
 MICO::CDRDecoder::get_ulonglongs (CORBA::ULongLong *p, CORBA::ULong l)
 {
-    if (!buf->ralign (8))
+    if (!buf->ralign (8) || !check_chunk ())
 	return FALSE;
 
     if (data_bo == mach_bo)
@@ -1995,14 +1946,26 @@ MICO::CDRDecoder::get_wstring (CORBA::WString_out s)
     CORBA::ULong len;
     if (!get_ulong (len))
         return FALSE;
-    if (len == 0 || len*sizeof(CORBA::WChar) > buf->length())
+    if (len == 0 || len*sizeof(CORBA::UShort) > buf->length())
       return FALSE;
 
     CORBA::WChar * p = CORBA::wstring_alloc (len-1);
-    if (!buf->get (p, len * sizeof (CORBA::WChar))) {
+    CORBA::UShort u;
+    CORBA::ULong i;
+
+    for (i=0; i<len; i++) {
+      if (!get_ushort (u)) {
+	CORBA::wstring_free (p);
+	return FALSE;
+      }
+      p[i] = u;
+    }
+
+    if (p[len-1] != 0) {
       CORBA::wstring_free (p);
       return FALSE;
     }
+
     s = p;
     return TRUE;
   }
