@@ -1,6 +1,6 @@
 /*
  *  MICO --- an Open Source CORBA implementation
- *  Copyright (c) 1997-2001 by The Mico Team
+ *  Copyright (c) 1997-2005 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -30,6 +30,9 @@
 #ifndef FAST_PCH
 
 #define MICO_CONF_IMR
+#define MICO_CONF_IR
+#define MICO_CONF_INTERCEPT
+#define MICO_CONF_POA
 
 #include <CORBA-SMALL.h>
 #include <string.h>
@@ -40,7 +43,6 @@
 #endif
 #include <mico/util.h>
 #include <mico/impl.h>
-#include <mico/intercept.h>
 #include <mico/template_impl.h>
 
 #include <mico/code_set_registry.h>
@@ -53,6 +55,7 @@ using namespace std;
 namespace CORBA {
   CORBA::Codeset *Codeset::_special[CORBA::Codeset::_SpecialMax] = { 0 };
   CORBA::Boolean Codeset::_disabled = FALSE;
+  CORBA::Boolean Codeset::_enabled_in_iiopprofile = FALSE;
 }
 
 CORBA::Codeset::Codeset()
@@ -571,7 +574,7 @@ CORBA::Long
 MICO::UniCodesetConv::encode (const CORBA::Char *f, CORBA::ULong len,
 			      CORBA::Buffer &t, CORBA::Boolean terminate)
 {
-    char *from, *to, *cp;
+    char *from = NULL, *to = NULL, *cp = NULL;
 
     CORBA::ULong flen = _from->codepoint_size();
     assert (flen == 1 || flen == 2 || flen == 4);
@@ -637,7 +640,7 @@ CORBA::Long
 MICO::UniCodesetConv::encode (const CORBA::WChar *f, CORBA::ULong len,
 			      CORBA::Buffer &t, CORBA::Boolean terminate)
 {
-    char *cp, *from;
+    char *cp = NULL, *from = NULL;
 
     CORBA::ULong flen = _from->codepoint_size();
     assert (flen == 1 || flen == 2 || flen == 4);
@@ -1911,13 +1914,17 @@ MICO::GIOP_1_2_CodeSetCoder::get_wstring (CORBA::DataDecoder & decoder,
       }
 
       buf = CORBA::wstring_alloc (length / 2);
-
+      // kcg: added here -- see comment few lines below
+      buf[length / 2] = 0;
       if (!swap) {
 	if (!decoder.buffer()->get (buf, length)) {
 	  CORBA::wstring_free (buf);
 	  return FALSE;
 	}
-	buf[length / 2] = 0;
+	// kcg: why we end string with '0' only when !swap?
+	// imho this is wrong, so I've commented it here
+	// and added it directly after wstring_alloc above
+	//buf[length / 2] = 0;
       }
       else {
 	CORBA::WChar * ptr = buf;
@@ -2551,6 +2558,8 @@ public:
 	opts["-ORBNativeCS"]   = "arg-expected";
 	opts["-ORBNativeWCS"]  = "arg-expected";
 	opts["-ORBNoCodeSets"] = "";
+	opts["-ORBCodeSetsInIIOPProfile"] = "";
+	opts["-ORBNoCodeSetsInMultiCompProfile"] = "";
 
 	MICOGetOpt opt_parser (opts);
 	CORBA::Boolean r = opt_parser.parse (orb->rcfile(), TRUE);
@@ -2561,6 +2570,8 @@ public:
 	const char *cs_name = 0;
 	const char *wcs_name = 0;
 	CORBA::Boolean disable = FALSE;
+	CORBA::Boolean iiop_profile_enable = FALSE;
+	CORBA::Boolean multi_comp_profile_disable = FALSE;
 
 	const MICOGetOpt::OptVec &o = opt_parser.opts();
 	MICOGetOpt::OptVec::const_iterator i;
@@ -2573,10 +2584,15 @@ public:
 		wcs_name = val.c_str();
 	    } else if (arg == "-ORBNoCodeSets") {
 		disable = TRUE;
+	    } else if (arg == "-ORBCodeSetsInIIOPProfile") {
+		iiop_profile_enable = TRUE;
+	    } else if (arg == "-ORBNoCodeSetsInMultiCompProfile") {
+		multi_comp_profile_disable = TRUE;
 	    }
 	}
 
 	CORBA::Codeset::disable (disable);
+	CORBA::Codeset::enable_in_iiop_profile(iiop_profile_enable);
 
 	// native char code set
 	if (!cs_name)
@@ -2666,7 +2682,7 @@ public:
 
 
 	// install code set info in ior template
-	if (!disable) {
+	if (!disable && !multi_comp_profile_disable) {
 	    MICO::MultiCompProfile *mcp =
 		new MICO::MultiCompProfile (CORBA::MultiComponent());
 
@@ -2682,12 +2698,122 @@ public:
     }
 } __cs_init;
 
+namespace MICO
+{
+    class CodesetIORInterceptor_impl
+	: virtual public PortableInterceptor::IORInterceptor,
+	  virtual public CORBA::LocalObject
+    {
+    public:
+	CodesetIORInterceptor_impl()
+	    : name_("CodesetIORInterceptor")
+	{}
+
+	virtual char*
+	name()
+	{ return CORBA::string_dup(name_.c_str()); }
+
+	virtual void
+	name(char* name)
+	{ name_ = name; }
+
+	virtual void
+	destroy()
+	{}
+
+	virtual void
+	establish_components(PortableInterceptor::IORInfo_ptr info);
+    private:
+	std::string name_;
+    };
+
+    class CodesetORBInitializer
+	: virtual public PortableInterceptor::ORBInitializer,
+	  virtual public CORBA::LocalObject
+    {
+    public:
+	CodesetORBInitializer();
+
+	virtual
+	~CodesetORBInitializer();
+
+	virtual void
+	pre_init(PortableInterceptor::ORBInitInfo_ptr info);
+
+	virtual void
+	post_init(PortableInterceptor::ORBInitInfo_ptr info);
+    };
+} // MICO
+
+void
+MICO::CodesetIORInterceptor_impl::establish_components(PortableInterceptor::IORInfo_ptr info)
+{
+    if (CORBA::Codeset::enabled_in_iiop_profile()
+	&& !CORBA::Codeset::disabled()) {
+	//cerr << "enabled for IIOP profiles!" << endl;
+	CORBA::Buffer* buf = new CORBA::Buffer;
+	CDREncoder encoder(buf);
+	CodesetComponent comp
+	    (CORBA::Codeset::special_cs(CORBA::Codeset::NativeCS)->id(),
+	     CORBA::Codeset::special_cs(CORBA::Codeset::NativeWCS)->id(),
+	     vector<MICO::CodesetComponent::CodeSetId>(),
+	     vector<MICO::CodesetComponent::CodeSetId>());
+	comp.encode(encoder);
+	buf->rseek_beg(0);
+	IOP::TaggedComponent component;
+	component.tag = comp.id();
+	component.component_data.length(buf->length());
+	for (CORBA::ULong i=0; i<buf->length(); i++) {
+	    component.component_data[i] = (buf->data())[i];
+	}
+	try {
+	    info->add_ior_component_to_profile
+		(component, CORBA::IORProfile::TAG_INTERNET_IOP);
+	} catch (CORBA::BAD_PARAM& ex) {
+	    if (MICO::Logger::IsLogged(MICO::Logger::ORB))
+		MICO::Logger::Stream(MICO::Logger::ORB)
+		    << "IOR: can't find IIOP profile" << endl;
+	}
+	try {
+	    info->add_ior_component_to_profile
+		(component, CORBA::IORProfile::TAG_SSL_INTERNET_IOP);
+	} catch (CORBA::BAD_PARAM& ex) {
+	    if (MICO::Logger::IsLogged(MICO::Logger::ORB))
+		MICO::Logger::Stream(MICO::Logger::ORB)
+		    << "IOR: can't find SSL IIOP profile" << endl;
+	}
+    }
+}
+
+MICO::CodesetORBInitializer::CodesetORBInitializer()
+{
+}
+
+MICO::CodesetORBInitializer::~CodesetORBInitializer()
+{
+}
+
+void
+MICO::CodesetORBInitializer::pre_init
+(PortableInterceptor::ORBInitInfo_ptr info)
+{
+    PortableInterceptor::IORInterceptor_var icept = new CodesetIORInterceptor_impl;
+    info->add_ior_interceptor(icept);
+}
+
+void
+MICO::CodesetORBInitializer::post_init
+(PortableInterceptor::ORBInitInfo_ptr info)
+{
+}
+
 void
 CORBA::Codeset::_init ()
 {
   // Is it really needed? E.g. for some obsolete compilers?
   // In that case, please uncomment following line
   //__cs_init;
+  PortableInterceptor::register_orb_initializer(new MICO::CodesetORBInitializer());
 }
 
 struct __init_CODESET
