@@ -45,6 +45,7 @@
 
 #endif // FAST_PCH
 
+//#define USE_OLD_INTERCEPTORS
 
 using namespace std;
 
@@ -1747,8 +1748,11 @@ CORBA::StaticRequest::StaticRequest (CORBA::Object_ptr obj, const char *opname)
     _ctx_list = 0;
     _obj = CORBA::Object::_duplicate (obj);
 
+#ifdef USE_OLD_INTERCEPTORS
+#warning "using old interceptors!"
     _iceptreq = Interceptor::ClientInterceptor::_create_request
 	(_obj, _opname, *context(), CORBA::Request::_nil());
+#endif // USE_OLD_INTERCEPTORS
     _cri = PInterceptor::PI::_create_cri(_obj, _opname);
 }
 
@@ -1759,7 +1763,9 @@ CORBA::StaticRequest::~StaticRequest ()
     CORBA::release (_ctx);
     CORBA::release (_env);
     CORBA::release (_ctx_list);
+#ifdef USE_OLD_INTERCEPTORS
     CORBA::release (_iceptreq);
+#endif // USE_OLD_INTERCEPTORS
     CORBA::release (_cri);
 }
 
@@ -1854,8 +1860,10 @@ CORBA::StaticRequest::get_in_args (DataEncoder *ec)
 	    // If we don't do this, we will have a memory leak when
 	    // the return parameters overwrite the original input
 	    // parameter
-	    if (_args[i]->flags() & CORBA::ARG_INOUT)
-		_args[i]->release();
+
+	    // Release now done in set_out_args (DataDecoder *dc, Boolean is_ex)
+	    // if (_args[i]->flags() & CORBA::ARG_INOUT)
+	    //  _args[i]->release();
 	}
     }
 
@@ -2010,6 +2018,9 @@ CORBA::StaticRequest::set_out_args (CORBA::DataDecoder *dc,
 	    }
 	}
 	for (mico_vec_size_type i = 0; i < _args.size(); ++i) {
+	    if (_args[i]->flags() & CORBA::ARG_INOUT)
+		_args[i]->release();
+
 	    if (_args[i]->flags() & (CORBA::ARG_OUT|CORBA::ARG_INOUT)) {
 		ret = _args[i]->demarshal (*dc);
                 if (!ret) {
@@ -2133,27 +2144,32 @@ CORBA::StaticRequest::invoke ()
     PInterceptor::PI::_send_request_ip
 	(_cri, CORBA::ORB::get_msgid(id), _args, this->ctx_list(), this->ctx(),
 	 this->context());
+#ifdef USE_OLD_INTERCEPTORS
     if (_iceptreq && !Interceptor::ClientInterceptor::
 	_exec_initialize_request ((Interceptor::LWRequest_ptr)_iceptreq,
 				  env())) {
 	return;
     }
+#endif // USE_OLD_INTERCEPTORS
 
     id = orb->invoke_async (_obj, this, CORBA::Principal::_nil(),
 			    TRUE, 0, id);
 //      msgid = orb->invoke_async (_obj, this, CORBA::Principal::_nil(),
 //  			       TRUE, 0, msgid);// we got _msgid already
 
+#ifdef USE_OLD_INTERCEPTORS
     if (_iceptreq && !Interceptor::ClientInterceptor::
 	_exec_after_marshal ((Interceptor::LWRequest_ptr)_iceptreq,
 			     env())) {
 	orb->cancel (id);
 	return;
     }
+#endif // USE_OLD_INTERCEPTORS
     while (!done) {
 	CORBA::Boolean ret = orb->wait (id);
 	assert (ret);
 
+#ifdef USE_OLD_INTERCEPTORS
 	// XXX called multiple times in case of Forward
 	if (_iceptreq && !Interceptor::ClientInterceptor::
 	    _exec_before_unmarshal ((Interceptor::LWRequest_ptr)_iceptreq,
@@ -2161,6 +2177,7 @@ CORBA::StaticRequest::invoke ()
 	    orb->cancel (id);
 	    return;
 	}
+#endif // USE_OLD_INTERCEPTORS
 
 	GIOP::AddressingDisposition ad;
 	CORBA::InvokeStatus rs = orb->get_invoke_reply (id, obj, dummy, ad);
@@ -2231,7 +2248,7 @@ CORBA::StaticRequest::invoke ()
 	    // receive_exception (user exception)
 	    PInterceptor::PI::_receive_exception_ip
 		(_cri, PortableInterceptor::USER_EXCEPTION,
-		 this->exception()->_clone(), dummy->context()); // end point
+		 this->exception(), dummy->context()); // end point
 	    done = TRUE;
 	    break;
 
@@ -2239,7 +2256,7 @@ CORBA::StaticRequest::invoke ()
 	    // receive_exception (system exception)
 	    PInterceptor::PI::_receive_exception_ip
 		(_cri, PortableInterceptor::SYSTEM_EXCEPTION,
-		 this->exception()->_clone(), dummy->context());
+		 this->exception(), dummy->context());
 	    if (_obj->_is_forwarded()) {
 		/*
 		 * [15-44] says:
@@ -2284,21 +2301,25 @@ CORBA::StaticRequest::invoke ()
 	}
     }
 
+#ifdef USE_OLD_INTERCEPTORS
     if (_iceptreq) {
 	Interceptor::ClientInterceptor::
 	    _exec_finish_request ((Interceptor::LWRequest_ptr)_iceptreq,
 				  env());
     }
+#endif // USE_OLD_INTERCEPTORS
 }
 
 void
 CORBA::StaticRequest::oneway ()
 {
+#ifdef USE_OLD_INTERCEPTORS
     if (_iceptreq && !Interceptor::ClientInterceptor::
 	_exec_initialize_request ((Interceptor::LWRequest_ptr)_iceptreq,
 				  env())) {
 	return;
     }
+#endif // USE_OLD_INTERCEPTORS
 //      _obj->_orbnc()->invoke_async (_obj, this, CORBA::Principal::_nil(), FALSE);
 
     PInterceptor::PI::_send_request_ip
@@ -2313,17 +2334,19 @@ CORBA::StaticRequest::oneway ()
 	cerr << "static.cc hack pi...." << endl;
 	CORBA::OBJECT_NOT_EXIST ex;
 	PInterceptor::PI::_receive_exception_ip
-	    (_cri, PortableInterceptor::SYSTEM_EXCEPTION, ex._clone(),
+	    (_cri, PortableInterceptor::SYSTEM_EXCEPTION, ex,
 	     this->ctx_list(), this->ctx(), this->context());
     }
     else {
 	PInterceptor::PI::_receive_other_ip(_cri);
     }
+#ifdef USE_OLD_INTERCEPTORS
     if (_iceptreq) {
 	Interceptor::ClientInterceptor::
 	    _exec_after_marshal ((Interceptor::LWRequest_ptr)_iceptreq,
 				 env());
     }
+#endif // USE_OLD_INTERCEPTORS
 }
 
 void
@@ -2331,12 +2354,14 @@ CORBA::StaticRequest::send_deferred ()
 {
     CORBA::ORB_ptr orb = _obj->_orbnc();
 
+#ifdef USE_OLD_INTERCEPTORS
     if (_iceptreq && !Interceptor::ClientInterceptor::
 	_exec_initialize_request ((Interceptor::LWRequest_ptr)_iceptreq,
 				  env())) {
         _id = 0;
 	return;
     }
+#endif // USE_OLD_INTERCEPTORS
     
 //      CORBA::ULong msgid = orb->new_msgid();
     CORBA::ORBMsgId id = orb->new_orbid();
@@ -2349,12 +2374,14 @@ CORBA::StaticRequest::send_deferred ()
 			     TRUE, 0, id);
     PInterceptor::PI::_receive_other_ip(_cri);
 
+#ifdef USE_OLD_INTERCEPTORS
     if (_iceptreq && !Interceptor::ClientInterceptor::
 	_exec_after_marshal ((Interceptor::LWRequest_ptr)_iceptreq,
 			     env())) {
 	orb->cancel (_id);
 	_id = 0;
     }
+#endif // USE_OLD_INTERCEPTORS
 }
 
 void
@@ -2375,6 +2402,7 @@ CORBA::StaticRequest::get_response ()
 	CORBA::Boolean ret = orb->wait (_id);
 	assert (ret);
 
+#ifdef USE_OLD_INTERCEPTORS
 	// XXX called multiple times in case of Forward
 	if (_iceptreq && !Interceptor::ClientInterceptor::
 	    _exec_before_unmarshal ((Interceptor::LWRequest_ptr)_iceptreq,
@@ -2383,6 +2411,7 @@ CORBA::StaticRequest::get_response ()
             _id = 0;
 	    return;
 	}
+#endif // USE_OLD_INTERCEPTORS
 
 	CORBA::InvokeStatus rs = orb->get_invoke_reply (_id, obj, dummy, ad);
 //  	CORBA::InvokeStatus rs = orb->get_invoke_reply (_msgid, obj,
@@ -2455,7 +2484,7 @@ CORBA::StaticRequest::get_response ()
 	case CORBA::InvokeUsrEx:
 	    PInterceptor::PI::_receive_exception_ip
 		(_cri, PortableInterceptor::USER_EXCEPTION,
-		 this->exception()->_clone(), this->ctx_list(),
+		 this->exception(), this->ctx_list(),
 		 this->ctx(), dummy->context());
 	    done = TRUE;
 	    break;
@@ -2463,7 +2492,7 @@ CORBA::StaticRequest::get_response ()
 	case CORBA::InvokeSysEx:
 	    PInterceptor::PI::_receive_exception_ip
 		(_cri, PortableInterceptor::SYSTEM_EXCEPTION,
-		 this->exception()->_clone(), this->ctx_list(),
+		 this->exception(), this->ctx_list(),
 		 this->ctx(), dummy->context());
 	    if (_obj->_is_forwarded()) {
 		/*
@@ -2512,11 +2541,13 @@ CORBA::StaticRequest::get_response ()
 	}
     }
 
+#ifdef USE_OLD_INTERCEPTORS
     if (_iceptreq) {
 	Interceptor::ClientInterceptor::
 	    _exec_finish_request ((Interceptor::LWRequest_ptr)_iceptreq,
                                   env());
     }
+#endif // USE_OLD_INTERCEPTORS
     // kcg: Is this ok instead of delete _id; _id = 0; ??
     _id = 0;
 }
@@ -2525,35 +2556,6 @@ CORBA::StaticRequest::get_response ()
 //-----------------
 
 
-CORBA::StaticImplementation::StaticImplementation ()
-{
-}
-
-CORBA::StaticImplementation::~StaticImplementation ()
-{
-}
-
-CORBA::ServerRequestBase_ptr
-CORBA::StaticImplementation::make_request (ORBRequest *req,
-					   CORBA::Object_ptr obj,
-					   CORBA::ORBMsgId id,
-					   CORBA::ObjectAdapter *oa,
-					   CORBA::Principal_ptr pr)
-{
-    return new StaticServerRequest (req, obj, id, oa, pr);
-}
-
-void
-CORBA::StaticImplementation::doinvoke (ServerRequestBase_ptr req,
-				       Environment &env)
-{
-    invoke ((StaticServerRequest_ptr)req, env);
-}
-
-
-//-----------------
-
-
 CORBA::StaticServerRequest::StaticServerRequest (ORBRequest *req,
 						 Object_ptr obj, CORBA::ORBMsgId id,
 						 CORBA::ObjectAdapter *oa,
@@ -2563,8 +2565,10 @@ CORBA::StaticServerRequest::StaticServerRequest (ORBRequest *req,
       _req (ORBRequest::_duplicate (req)),
       _res (0), _id (id), _aborted (FALSE)
 {
+#ifdef USE_OLD_INTERCEPTORS
     _iceptreq = Interceptor::ServerInterceptor::_create_request
 	(_obj, req->op_name(), *req->context(), CORBA::ServerRequest::_nil());
+#endif // USE_OLD_INTERCEPTORS
 }
 
 CORBA::StaticServerRequest::~StaticServerRequest ()
@@ -2588,17 +2592,21 @@ CORBA::StaticServerRequest::~StaticServerRequest ()
 //          _oa->answer_invoke (_msgid, _obj, _req, stat);
         _oa->answer_invoke (_id, _obj, _req, stat);
 
+#ifdef USE_OLD_INTERCEPTORS
         if (_iceptreq && !_aborted && !Interceptor::ServerInterceptor::
             _exec_finish_request ((Interceptor::LWServerRequest_ptr)_iceptreq,
                                   &_env)) {
             // XXX how to abort request here ??
         }
+#endif // USE_OLD_INTERCEPTORS
     }
 
     CORBA::release (_obj);
     CORBA::release (_ctx);
     CORBA::release (_req);
+#ifdef USE_OLD_INTERCEPTORS
     CORBA::release (_iceptreq);
+#endif // USE_OLD_INTERCEPTORS
 }
 
 void
@@ -2630,6 +2638,7 @@ CORBA::StaticServerRequest::read_args ()
      * add_*_arg() in order to read arguments from ORBRequest
      */
 
+#ifdef USE_OLD_INTERCEPTORS
     if (_iceptreq && !Interceptor::ServerInterceptor::
 	_exec_initialize_request ((Interceptor::LWServerRequest_ptr)_iceptreq,
 				  &_env)) {
@@ -2637,6 +2646,7 @@ CORBA::StaticServerRequest::read_args ()
 	write_results ();
 	return FALSE;
     }
+#endif // USE_OLD_INTERCEPTORS
 
     if (!_req->get_in_args (&_args, _ctx)) {
         if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
@@ -2650,6 +2660,7 @@ CORBA::StaticServerRequest::read_args ()
         return FALSE;
     }
 
+#ifdef USE_OLD_INTERCEPTORS
     if (_iceptreq && !Interceptor::ServerInterceptor::
 	_exec_after_unmarshal ((Interceptor::LWServerRequest_ptr)_iceptreq,
 			       &_env)) {
@@ -2657,6 +2668,7 @@ CORBA::StaticServerRequest::read_args ()
 	write_results ();
 	return FALSE;
     }
+#endif // USE_OLD_INTERCEPTORS
     
     CORBA::ORB_ptr orb = _obj->_orbnc();
     CORBA::ORBInvokeRec* rec = _id;
@@ -2701,11 +2713,13 @@ CORBA::StaticServerRequest::write_results ()
      * is left in order to write back results into ORBRequest ...
      */
 
+#ifdef USE_OLD_INTERCEPTORS
     if (_iceptreq && !_aborted && !Interceptor::ServerInterceptor::
 	_exec_before_marshal ((Interceptor::LWServerRequest_ptr)_iceptreq,
 			      &_env)) {
 	_aborted = TRUE;
     }
+#endif // USE_OLD_INTERCEPTORS
 
     if (_env.exception()) {
 	_req->set_out_args (_env.exception());
