@@ -1,5 +1,5 @@
 #define FORCE_MARSHALLING
-
+#define MICO_CONF_POA
 #include <CORBA-SMALL.h>
 #include "attribute.h"
 #ifdef HAVE_ANSI_CPLUSPLUS_HEADERS
@@ -11,7 +11,7 @@
 
 using namespace std;
 
-class foo_impl : virtual public foo_skel
+class foo_impl : virtual public POA_foo
 {
 private:
   CORBA::Long a1;
@@ -41,9 +41,14 @@ int main( int argc, char *argv[] )
 {
   // ORB initialization
   CORBA::ORB_var orb = CORBA::ORB_init( argc, argv, "mico-local-orb" );
+  CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
+  PortableServer::POA_var poa = PortableServer::POA::_narrow(obj);
+  PortableServer::POAManager_var mgr = poa->the_POAManager();
+  mgr->activate();
 
   // server side
-  foo_impl* foo_obj = new foo_impl;
+  foo_impl* foo = new foo_impl;
+  foo_ptr foo_obj = foo->_this();
   
   CORBA::String_var foo_ref = orb->object_to_string( foo_obj );
   
@@ -51,9 +56,9 @@ int main( int argc, char *argv[] )
   
   // client side
 #ifdef FORCE_MARSHALLING
-  CORBA::Object_var obj = new CORBA::Object( new CORBA::IOR( foo_ref ) );
+  obj = new CORBA::Object( new CORBA::IOR( foo_ref ) );
 #else
-  CORBA::Object_var obj = orb->string_to_object( foo_ref );
+  obj = orb->string_to_object( foo_ref );
 #endif
   foo_var foo_client = foo::_narrow( obj );
 
